from django.utils.cache import patch_vary_headers

try:
    # Added in Django 1.10
    from django.utils.deprecation import MiddlewareMixin
except ImportError:
    _base_class = object  # pragma: no cover
else:
    _base_class = MiddlewareMixin  # pragma: no cover


class DoNotTrackMiddleware(_base_class):

    def process_request(self, request):
        """
        Sets flag request.DNT based on DNT HTTP header.
        """
        if 'HTTP_DNT' in request.META and request.META['HTTP_DNT'] == '1':
            request.DNT = True
        else:
            request.DNT = False

    def process_response(self, request, response):
        """
        Adds a "Vary" header for DNT, useful for caching.
        """
        patch_vary_headers(response, ['DNT'])

        return response



import django


SECRET_KEY = "dnt-tests"
DEBUG = True
ALLOWED_HOSTS = []

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    'testapp',
)

if django.VERSION[:2] < (1, 10):
    # Django 1.9 and earlier
    MIDDLEWARE_CLASSES = (
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.middleware.common.CommonMiddleware',
        'django.middleware.csrf.CsrfViewMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
        'django.middleware.clickjacking.XFrameOptionsMiddleware',
        'django.middleware.security.SecurityMiddleware',
        'dnt.middleware.DoNotTrackMiddleware',
    )
else:
    # Django 1.10 and later
    MIDDLEWARE = [
        'django.middleware.security.SecurityMiddleware',
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.middleware.common.CommonMiddleware',
        'django.middleware.csrf.CsrfViewMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
        'django.middleware.clickjacking.XFrameOptionsMiddleware',
        'dnt.middleware.DoNotTrackMiddleware',
    ]

ROOT_URLCONF = 'testapp.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:'
    }
}



# -*- coding: utf-8 -*-
"""Middleware tests."""
from __future__ import unicode_literals

from django.test import TestCase
from django.http import HttpRequest, HttpResponse

from dnt.middleware import DoNotTrackMiddleware


class DoNotTrackMiddlewareTest(TestCase):
    """Unit tests for the DoNotTrackMiddleware."""

    def request(self):
        """Create a request object for middleware testing."""
        req = HttpRequest()
        req.META = {
            'SERVER_NAME': 'testserver',
            'SERVER_PORT': 80,
        }
        req.path = req.path_info = "/"
        return req

    def response(self):
        """Create a response object for middleware testing."""
        resp = HttpResponse()
        resp.status_code = 200
        resp.content = b'    '
        return resp

    def test_request_dnt(self):
        """The header "DNT: 1" sets request.DNT."""
        request = self.request()
        request.META['HTTP_DNT'] = '1'
        DoNotTrackMiddleware().process_request(request)
        self.assertTrue(request.DNT)

    def test_request_dnt_off(self):
        """The header "DNT: 0" clears request.DNT."""
        request = self.request()
        request.META['HTTP_DNT'] = '0'
        DoNotTrackMiddleware().process_request(request)
        self.assertFalse(request.DNT)

    def test_request_no_dnt(self):
        """If the DNT header is not present, request.DNT is false."""
        request = self.request()
        DoNotTrackMiddleware().process_request(request)
        self.assertFalse(request.DNT)

    def test_response(self):
        """The Vary caching header in the response includes DNT."""
        request = self.request()
        response = self.response()
        response = DoNotTrackMiddleware().process_response(request, response)
        self.assertEqual(response['Vary'], 'DNT')



# -*- coding: utf-8 -*-
"""Test an application using django-dnt."""
from __future__ import unicode_literals

from django.test import TestCase


class AppTest(TestCase):
    """Test middleware through the Django test client."""

    def test_request_dnt(self):
        """Test a request with header "DNT: 1"."""
        response = self.client.get('/', HTTP_DNT='1')
        self.assertEqual(response['Vary'], 'DNT')
        content = response.content.decode('utf8')
        self.assertInHTML('<code class="dnt">True</code>', content)

    def test_request_no_dnt(self):
        """Test a request with no DNT header."""
        response = self.client.get('/')
        self.assertEqual(response['Vary'], 'DNT')
        content = response.content.decode('utf8')
        self.assertInHTML('<code class="dnt">False</code>', content)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Packaging setup for django-dnt."""

from setuptools import setup

description = 'Make Django requests aware of the DNT header'


def read_file(path):
    contents = open(path).read()
    if hasattr(contents, 'decode'):
        return contents.decode('utf8')  # Python2 bytes to unicode
    else:
        return contents  # Python3 reads unicode


def long_description():
    """Create a PyPI long description from docs."""
    readme = read_file('README.rst')
    body_tag = ".. Omit badges from docs"
    try:
        readme_body_start = readme.index(body_tag)
    except ValueError:
        readme_body = readme
    else:
        # Omit the badges and reconstruct the title
        readme_text = readme[readme_body_start + len(body_tag) + 1:]
        readme_body = """\
%(title_mark)s
%(title)s
%(title_mark)s
%(readme_text)s
""" % {
            'title_mark': '=' * len(description),
            'title': description,
            'readme_text': readme_text,
        }

    try:
        history = read_file('HISTORY.rst')
    except IOError:
        history = ''

    long_description = """\
%(readme)s

%(history)s
""" % {
        'readme': readme_body,
        'history': history
    }
    return long_description


setup(
    name='django-dnt',
    version='0.2.0',
    description=description + '.',
    long_description=long_description(),
    author='James Socol',
    author_email='james@mozilla.com',
    url='http://github.com/mozilla/django-dnt',
    license='BSD',
    packages=['dnt'],
    zip_safe=False,
    keywords='django-dnt dnt do not track',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Environment :: Web Environment :: Mozilla',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ]
)
